@@ -43,7 +43,7 @@ static const wchar *AllocCmdParam(const wchar *CmdLine,wchar **Par)
 }
 
 
-#if !defined(SFX_MODULE) && !defined(_ANDROID)
+#if !defined(SFX_MODULE)
 void CommandData::ParseCommandLine(bool Preprocess,int argc, char *argv[])
 {
   *Command=0;
@@ -85,7 +85,7 @@ void CommandData::ParseCommandLine(bool Preprocess,int argc, char *argv[])
 #endif
 
 
-#if !defined(SFX_MODULE) && !defined(_ANDROID)
+#if !defined(SFX_MODULE)
 void CommandData::ParseArg(wchar *Arg)
 {
   if (IsSwitch(*Arg) && !NoMoreSwitches)
@@ -99,14 +99,12 @@ void CommandData::ParseArg(wchar *Arg)
       wcsncpy(Command,Arg,ASIZE(Command));
 
 
-#ifndef GUI
       *Command=toupperw(*Command);
       // 'I' and 'S' commands can contain case sensitive strings after
       // the first character, so we must not modify their case.
       // 'S' can contain SFX name, which case is important in Unix.
       if (*Command!='I' && *Command!='S')
         wcsupper(Command);
-#endif
     }
     else
       if (*ArcName==0)
@@ -137,7 +135,7 @@ void CommandData::ParseArg(wchar *Arg)
 
               RAR_CHARSET Charset=FilelistCharset;
 
-#if defined(_WIN_ALL) && !defined(GUI)
+#if defined(_WIN_ALL)
               // for compatibility reasons we use OEM encoding
               // in Win32 console version by default
 
@@ -177,7 +175,7 @@ void CommandData::ParseDone()
 }
 
 
-#if !defined(SFX_MODULE) && !defined(_ANDROID)
+#if !defined(SFX_MODULE)
 void CommandData::ParseEnvVar()
 {
   char *EnvStr=getenv("RAR");
@@ -192,7 +190,7 @@ void CommandData::ParseEnvVar()
 
 
 
-#if !defined(SFX_MODULE) && !defined(_ANDROID)
+#if !defined(SFX_MODULE)
 // Preprocess those parameters, which must be processed before the rest of
 // command line. Return 'false' to stop further processing.
 void CommandData::PreprocessArg(const wchar *Arg)
@@ -204,7 +202,6 @@ void CommandData::PreprocessArg(const wchar *Arg)
       NoMoreSwitches=true;
     if (wcsicomp(Arg,L"cfg-")==0)
       ConfigDisabled=true;
-#ifndef GUI
     if (wcsnicomp(Arg,L"ilog",4)==0)
     {
       // Ensure that correct log file name is already set
@@ -212,15 +209,12 @@ void CommandData::PreprocessArg(const wchar *Arg)
       ProcessSwitch(Arg);
       InitLogOptions(LogName,ErrlogCharset);
     }
-#endif
     if (wcsnicomp(Arg,L"sc",2)==0)
     {
       // Process -sc before reading any file lists.
       ProcessSwitch(Arg);
-#ifndef GUI
       if (*LogName!=0)
         InitLogOptions(LogName,ErrlogCharset);
-#endif
     }
   }
   else
@@ -230,7 +224,7 @@ void CommandData::PreprocessArg(const wchar *Arg)
 #endif
 
 
-#if !defined(GUI) && !defined(SFX_MODULE) && !defined(_ANDROID)
+#if !defined(SFX_MODULE)
 void CommandData::ReadConfig()
 {
   StringList List;
@@ -265,7 +259,7 @@ void CommandData::ReadConfig()
 #endif
 
 
-#if !defined(SFX_MODULE) && !defined(_ANDROID)
+#if !defined(SFX_MODULE)
 void CommandData::ProcessSwitchesString(const wchar *Str)
 {
   wchar *Par;
@@ -279,7 +273,7 @@ void CommandData::ProcessSwitchesString(const wchar *Str)
 #endif
 
 
-#if !defined(SFX_MODULE) && !defined(_ANDROID)
+#if !defined(SFX_MODULE)
 void CommandData::ProcessSwitch(const wchar *Switch)
 {
 
@@ -583,7 +577,7 @@ void CommandData::ProcessSwitch(const wchar *Switch)
         {
           RAR_CHARSET Charset=FilelistCharset;
 
-#if defined(_WIN_ALL) && !defined(GUI)
+#if defined(_WIN_ALL)
           // for compatibility reasons we use OEM encoding
           // in Win32 console version by default
 
@@ -760,6 +754,9 @@ void CommandData::ProcessSwitch(const wchar *Switch)
                 case 'U':
                   rch=RCH_UNICODE;
                   break;
+                case 'F':
+                  rch=RCH_UTF8;
+                  break;
                 default :
                   BadSwitch(Switch);
                   AlreadyBad=true;
@@ -818,6 +815,8 @@ void CommandData::ProcessSwitch(const wchar *Switch)
             bool CommonMode=Switch[2]>='0' && Switch[2]<='4';
             if (CommonMode)
               Mode=(EXTTIME_MODE)(Switch[2]-'0');
+            if (Mode==EXTTIME_HIGH1 || Mode==EXTTIME_HIGH2) // '2' and '3' not supported anymore.
+              Mode=EXTTIME_HIGH3;
             if (Switch[2]=='-')
               Mode=EXTTIME_NONE;
             if (CommonMode || Switch[2]=='-' || Switch[2]=='+' || Switch[2]==0)
@@ -826,6 +825,8 @@ void CommandData::ProcessSwitch(const wchar *Switch)
             {
               if (Switch[3]>='0' && Switch[3]<='4')
                 Mode=(EXTTIME_MODE)(Switch[3]-'0');
+              if (Mode==EXTTIME_HIGH1 || Mode==EXTTIME_HIGH2) // '2' and '3' not supported anymore.
+                Mode=EXTTIME_HIGH3;
               if (Switch[3]=='-')
                 Mode=EXTTIME_NONE;
               switch(toupperw(Switch[2]))
@@ -888,19 +889,15 @@ void CommandData::ProcessSwitch(const wchar *Switch)
     case 'Z':
       if (Switch[1]==0)
       {
-#ifndef GUI // stdin is not supported by WinRAR.
         // If comment file is not specified, we read data from stdin.
         wcscpy(CommentFile,L"stdin");
-#endif
       }
       else
         wcsncpyz(CommentFile,Switch+1,ASIZE(CommentFile));
       break;
-#ifndef GUI
     case '?' :
       OutHelp(RARX_SUCCESS);
       break;
-#endif
     default :
       BadSwitch(Switch);
       break;
@@ -909,7 +906,7 @@ void CommandData::ProcessSwitch(const wchar *Switch)
 #endif
 
 
-#if !defined(SFX_MODULE) && !defined(_ANDROID)
+#if !defined(SFX_MODULE)
 void CommandData::BadSwitch(const wchar *Switch)
 {
   mprintf(St(MUnknownOption),Switch);
@@ -918,7 +915,6 @@ void CommandData::BadSwitch(const wchar *Switch)
 #endif
 
 
-#ifndef GUI
 void CommandData::OutTitle()
 {
   if (BareOutput || DisableCopyright)
@@ -954,11 +950,25 @@ void CommandData::OutTitle()
 #ifdef UNRAR
   mprintf(St(MUCopyright),Version,RARVER_YEAR);
 #else
+#ifndef FREE_UNRAR // To remove this code from unrar source.
+  mprintf(St(MCopyright),Version,RARVER_YEAR,RARVER_DAY,GetMonthName(RARVER_MONTH-1),RARVER_YEAR);
+
+  wchar RegName[ASIZE(Reg.RegName)];
+  ConvertUtfRegString(Reg.RegName,NULL,RegName,ASIZE(RegName));
+
+  if (RegVer)
+    mprintf(St(MRegTo),RegName);
+  else
+  {
+    mprintf(St(MShare));
+    if (*Reg.ErrInfo!=0)
+      mprintf(Reg.ErrInfo);
+  }
+#endif
 #endif
 #endif
 #endif
 }
-#endif
 
 
 inline bool CmpMSGID(MSGID i1,MSGID i2)
@@ -975,13 +985,13 @@ inline bool CmpMSGID(MSGID i1,MSGID i2)
 
 void CommandData::OutHelp(RAR_EXIT ExitCode)
 {
-#if !defined(GUI) && !defined(SILENT)
+#if !defined(SILENT)
   OutTitle();
   static MSGID Help[]={
 #ifdef SFX_MODULE
     // Console SFX switches definition.
     MCHelpCmd,MSHelpCmdE,MSHelpCmdT,MSHelpCmdV
-#elif defined(UNRAR)
+#else
     // UnRAR switches definition.
     MUNRARTitle1,MRARTitle2,MCHelpCmd,MCHelpCmdE,MCHelpCmdL,
     MCHelpCmdP,MCHelpCmdT,MCHelpCmdV,MCHelpCmdX,MCHelpSw,MCHelpSwm,
@@ -994,7 +1004,6 @@ void CommandData::OutHelp(RAR_EXIT ExitCode)
     MCHelpSwTA,MCHelpSwTB,MCHelpSwTN,MCHelpSwTO,MCHelpSwTS,MCHelpSwU,
     MCHelpSwVUnr,MCHelpSwVER,MCHelpSwVP,MCHelpSwX,MCHelpSwXa,MCHelpSwXal,
     MCHelpSwY
-#else
 #endif
   };
 
@@ -1227,12 +1236,15 @@ int CommandData::IsProcessFile(FileHeader &FileHead,bool *ExactMatch,int MatchTy
 }
 
 
-#ifndef GUI
 void CommandData::ProcessCommand()
 {
 #ifndef SFX_MODULE
 
+#if defined(ALLOW_UNLOCK) && !defined(UNRAR)
+  const wchar *SingleCharCommands=L"FUADPXET";
+#else
   const wchar *SingleCharCommands=L"FUADPXETK";
+#endif
   if (Command[0]!=0 && Command[1]!=0 && wcschr(SingleCharCommands,Command[0])!=NULL || *ArcName==0)
     OutHelp(*Command==0 ? RARX_SUCCESS:RARX_USERERROR); // Return 'success' for 'rar' without parameters.
 
@@ -1273,6 +1285,100 @@ void CommandData::ProcessCommand()
 
   switch(Command[0])
   {
+#ifndef UNRAR
+    case 'A':
+    case 'F':
+    case 'U':
+    case 'M':
+      {
+        if (*Command=='F')
+          FreshFiles=true;
+        if (*Command=='U')
+          UpdateFiles=true;
+        CmdAdd Add(this);
+        Add.DoAdd();
+      }
+      break;
+    case 'D':
+      {
+        CmdAdd Delete(this);
+        Delete.DoDelete();
+      }
+      break;
+    case 'S':
+      CmdSFX(this);
+      break;
+    case 'C':
+      switch(Command[1])
+      {
+        case 0:
+#ifndef GUI // stdin is not supported by WinRAR.
+          if (*CommentFile==0)
+            wcscpy(CommentFile,L"stdin");
+#endif
+          ProcessArchive(this);
+          break;
+/*
+        case 'F':
+          AddFileComment(this);
+          break;
+*/
+        case 'W':
+          WriteComment(this);
+          break;
+        case 'H':
+          ProcessArchive(this);
+          break;
+        default:
+          OutHelp(RARX_USERERROR);
+      }
+      break;
+    case 'R':
+      switch(Command[1])
+      {
+        case 'R':
+          Recovery=GetRecoverySize(Command+2,DEFAULT_RECOVERY);
+          ProcessArchive(this);
+          break;
+        case 'V':
+          {
+            RecVolNumber=GetRecoverySize(Command+2,DEFAULT_RECVOLUMES);
+            wchar ArcName[NM];
+            if (GetArcName(ArcName,ASIZE(ArcName)))
+              RecVolumesMake(this,ArcName);
+          }
+          break;
+        case 'C':
+          {
+            wchar ArcName[NM];
+            if (GetArcName(ArcName,ASIZE(ArcName)))
+              RecVolumesRestore(this,ArcName,false);
+          }
+          break;
+        case 'N':
+          PrepareRenameArgs(this);
+          CmdRename(this);
+          break;
+        case 0:
+          CmdRepair(this);
+          break;
+        default:
+          OutHelp(RARX_USERERROR);
+          break;
+      }
+      break;
+    case 'K':
+#ifdef ALLOW_UNLOCK
+      if (Command[1]=='-')
+        Unlock=true;
+      else
+        Lock=true;
+#else
+      Lock=true;
+#endif
+      ProcessArchive(this);
+      break;
+#endif
     case 'P':
     case 'X':
     case 'E':
@@ -1295,7 +1401,6 @@ void CommandData::ProcessCommand()
   if (!BareOutput)
     mprintf(L"\n");
 }
-#endif
 
 
 void CommandData::AddArcName(const wchar *Name)
@@ -1313,9 +1418,9 @@ bool CommandData::GetArcName(wchar *Name,int MaxSize)
 bool CommandData::IsSwitch(int Ch)
 {
 #if defined(_WIN_ALL) || defined(_EMX)
-  return(Ch=='-' || Ch=='/');
+  return Ch=='-' || Ch=='/';
 #else
-  return(Ch=='-');
+  return Ch=='-';
 #endif
 }
 
@@ -1324,7 +1429,7 @@ bool CommandData::IsSwitch(int Ch)
 uint CommandData::GetExclAttr(const wchar *Str)
 {
   if (IsDigit(*Str))
-    return(wcstol(Str,NULL,0));
+    return wcstol(Str,NULL,0);
 
   uint Attr=0;
   while (*Str!=0)
