@@ -70,7 +70,7 @@ bool FindFile::Next(FindData *fd,bool GetSymLink)
       continue;
     wchar Name[NM];
     if (!CharToWide(ent->d_name,Name,ASIZE(Name)))
-      Log(NULL,St(MInvalidName),Name);
+      uiMsg(UIERROR_INVALIDNAME,UINULL,Name);
 
     if (CmpName(FindMask,Name,MATCH_NAMES))
     {
@@ -79,16 +79,13 @@ bool FindFile::Next(FindData *fd,bool GetSymLink)
       *PointToName(FullName)=0;
       if (wcslen(FullName)+wcslen(Name)>=ASIZE(FullName)-1)
       {
-#ifndef SILENT
-        Log(NULL,L"\n%ls%ls",FullName,Name);
-        Log(NULL,St(MPathTooLong));
-#endif
+        uiMsg(UIERROR_PATHTOOLONG,FullName,L"",Name);
         return false;
       }
       wcscat(FullName,Name);
       if (!FastFind(FullName,fd,GetSymLink))
       {
-        ErrHandler.OpenErrorMsg(NULL,FullName);
+        ErrHandler.OpenErrorMsg(FullName);
         continue;
       }
       wcscpy(fd->Name,FullName);
@@ -145,9 +142,17 @@ bool FindFile::FastFind(const wchar *FindMask,FindData *fd,bool GetSymLink)
     }
   fd->FileAttr=st.st_mode;
   fd->Size=st.st_size;
-  fd->mtime=st.st_mtime;
-  fd->atime=st.st_atime;
-  fd->ctime=st.st_ctime;
+
+#ifdef UNIX_TIME_NS
+  fd->mtime.SetUnixNS(st.st_mtim.tv_sec*(uint64)1000000000+st.st_mtim.tv_nsec);
+  fd->atime.SetUnixNS(st.st_atim.tv_sec*(uint64)1000000000+st.st_atim.tv_nsec);
+  fd->ctime.SetUnixNS(st.st_ctim.tv_sec*(uint64)1000000000+st.st_ctim.tv_nsec);
+#else
+  fd->mtime.SetUnix(st.st_mtime);
+  fd->atime.SetUnix(st.st_atime);
+  fd->ctime.SetUnix(st.st_ctime);
+#endif
+
   wcsncpyz(fd->Name,FindMask,ASIZE(fd->Name));
 #endif
   fd->Flags=0;
@@ -161,7 +166,7 @@ bool FindFile::FastFind(const wchar *FindMask,FindData *fd,bool GetSymLink)
 #ifdef _WIN_ALL
 HANDLE FindFile::Win32Find(HANDLE hFind,const wchar *Mask,FindData *fd)
 {
-  WIN32_FIND_DATAW FindData;
+  WIN32_FIND_DATA FindData;
   if (hFind==INVALID_HANDLE_VALUE)
   {
     hFind=FindFirstFile(Mask,&FindData);
@@ -195,9 +200,9 @@ HANDLE FindFile::Win32Find(HANDLE hFind,const wchar *Mask,FindData *fd)
     fd->ftCreationTime=FindData.ftCreationTime;
     fd->ftLastAccessTime=FindData.ftLastAccessTime;
     fd->ftLastWriteTime=FindData.ftLastWriteTime;
-    fd->mtime=FindData.ftLastWriteTime;
-    fd->ctime=FindData.ftCreationTime;
-    fd->atime=FindData.ftLastAccessTime;
+    fd->mtime.SetWinFT(&FindData.ftLastWriteTime);
+    fd->ctime.SetWinFT(&FindData.ftCreationTime);
+    fd->atime.SetWinFT(&FindData.ftLastAccessTime);
 
 
   }
